<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 01/28/2004 *
 * ===================                          Last change: 06/10/2004 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : what-list_refs.php                               *
 * -------------------------------------------------------------------- *
 * Short description : Show all referrals made by a member              *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Alle generierten Refs eines Mitgliedes anzeigen  *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2012 by Mailer Developer Team                   *
 * For more information visit: http://mxchange.org                      *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if ((!defined('__SECURITY')) || (!isAdmin())) {
	die();
} // END - if

// Add description as navigation point
addYouAreHereLink('admin', __FILE__);

if (isGetRequestElementSet('userid')) {
	// Secure userid
	$userid = bigintval(getRequestElement('userid'));

	// Fix missing variable
	$result_levels = FALSE;

	// User found?
	if (fetchUserData($userid)) {
		// Get total refs
		$totalReferrals       = getUsersTotalReferrals($userid);
		$totalLockedReferrals = getUsersTotalLockedReferrals($userid);

		// Output info message
		displayMessage(sprintf(getMessage('ADMIN_USER_TOTAL_REFS'), generateUserProfileLink(getRequestElement('userid')), $totalReferrals, $totalLockedReferrals));

		// Load all referral levels
		$result_levels = SQL_QUERY("SELECT
	`level`,
	`percents`
FROM
	`{?_MYSQL_PREFIX?}_refdepths`
WHERE
	`level` > 0
ORDER BY
	`level` ASC", __FILE__, __LINE__);

		// Are there some levels (VERY BAD IF NONE!)
		if (!SQL_HASZERONUMS($result_levels)) {
			// List all ref levels or entries if no refback is installed
			$OUT = '';
			while ($levels = SQL_FETCHARRAY($result_levels)) {
				// Load all refs of this user
				$result_refs = SQL_QUERY_ESC("SELECT `refid` FROM `{?_MYSQL_PREFIX?}_user_refs` WHERE `userid`=%s AND `level`=%s ORDER BY `refid` ASC",
					array(
						$userid,
						$levels['level']
					), __FILE__, __LINE__);
				//* DEBUG: */ debugOutput($userid.'/'.$levels['level'].'/'.SQL_NUMROWS($result_refs));

				// Is there levels?
				if (!SQL_HASZERONUMS($result_refs)) {
					// Count entries
					$totalReferrals       = getUsersTotalReferrals($userid, $levels['level']);
					$totalLockedReferrals = getUsersTotalLockedReferrals($userid, $levels['level']);

					// Output info message
					$levels['info'] = sprintf(getMessage('ADMIN_USER_TOTAL_REFS'), generateUserProfileLink($userid), $totalReferrals, $totalLockedReferrals);

					// Init variables
					$OUT_REFS = '';

					// Load all refs
					while ($content = SQL_FETCHARRAY($result_refs)) {
						// Is the data there?
						if (fetchUserData($content['refid'])) {
							// Check for referrals
							$refs_cnt = countSumTotalData(getUserData('userid'), 'user_refs', 'id', 'userid', TRUE);

							// Prepare data for the template
							// @TODO Try to rewrite some to EL
							$content = array(
								'userid'     => getUserData('userid'),
								'refs_link'  => '0',
								'email'      => '[<a href="' . generateEmailLink(getUserData('email'), 'user_data') . '">' . getUserData('email') . '</a>]',
								'registered' => generateDateTime(getUserData('joined'), 3),
							);

							// Check if referral count is larger 0 and update link
							if ($refs_cnt > 0) {
								$content['refs_link'] = generateUserProfileLink(getUserData('userid'), $refs_cnt, 'list_refs');
							} // END - if

							// Load template for level one
							$OUT_REFS .= loadTemplate('admin_list_refs_row', TRUE, $content);
						} else {
							// No refs found
							$OUT_REFS = loadTemplate('admin_list_refs_nodata', TRUE, $content);
						}

						// Add content
						$levels['rows'] = $OUT_REFS;
					} // END - while
				} else {
					// Output info message
					$levels['info'] = sprintf(getMessage('ADMIN_USER_TOTAL_REFS'), generateUserProfileLink($userid), 0, 0);

					// No refs found
					$levels['rows'] = loadTemplate('admin_list_refs_norefs', TRUE, array('userid' => $userid));
				}

				// Free result
				SQL_FREERESULT($result_refs);

				// Load level template
				$OUT .= loadTemplate('admin_list_refs_level', TRUE, $levels);
			} // END - while

			// Prepare content
			$content = array(
				'rows'   => $OUT,
				'userid' => getRequestElement('userid')
			);

			// Load main template
			loadTemplate('admin_list_refs', FALSE, $content);
		} else {
			// No refs made so far
			displayMessage('{--USER_REFERRAL_404--}');
		}

		// Free result
		SQL_FREERESULT($result_levels);
	} else {
		// User not found
		displayMessage('{%message,ADMIN_MEMBER_404=' . getRequestElement('userid') . '%}');
	}
} else {
	// Output selection form with all confirmed user accounts listed
	addMemberSelectionBox();
}

// [EOF]
?>
